function rizTruelle()
% Licence Creative Commons BY-NC-SA 4.0
% (Attribution, NonCommercial and ShareAlike).
% For more informations, see http://creativecommons.org/licenses/by-nc-sa/4.0/
%
% Authors : 
% Ivan Alouges       : Sound Design and History
% Matthieu Aussal    : Game design et programmation matlab
% David Cian         : Programmation C# 
% Florian Gouret     : Programmation C#
% Etienne Juan-Gesta : Programmation and validations  
% Pierre Lecouvey    : Graphisms
% Sharon Maillet     : Sound design
%
% Creation : 2016.01.31
%
% GAME RIZTRUELLE - Global Game Jam 2016 


% Cleaning
clear all
close all
clc
clf
delete(timerfindall)

% LEVEL DIFFICULTY : SUMO ANGRY (easy = 50, medium = 40, hard = 30);
tMax = 40;

% Variables globales pour echanges 
global KEYcuil
global KEYfire
global KEYgarb
global KEYtemp
global Hrice
global Htir
global Irice
global Fwind

% VARIABLES GLOBALES AUDIO
global BONUS
global CORDE
global DEFAITE
global GARGOUIL
global POUBouv
global POUBfer
global SUMOmange
global VICTORY

% LECTURE DES FICHIERS AUDIO
BONUS     = audioread('bonus.mp3');
CORDE     = audioread('corde.mp3');
DEFAITE   = audioread('defaite.mp3');
GARGOUIL  = audioread('gargouilli.mp3');
POUBouv   = audioread('poubelleOuverte.mp3');
POUBfer   = audioread('poubelleFermee.mp3');
SUMOmange = audioread('sumoMange.mp3');
VICTORY   = audioread('victoire.mp3');


%% INTRODUCTION
% Fenetre graphique
figure(1); clf;
set(gcf,'Name','RIZTRUELLE','Position',[600 200 1200 700],...
    'MenuBar','none','ToolBar','none','Color','w');

% Musique d'intro
[y,fs] = audioread('intro.mp3');
p = audioplayer(y,fs);
play(p);

% Slides d'intros
axes('position',[0 0 1 1]);
for i = 1:5
    data = imread(['diapo',num2str(i),'.png']);
    imagesc(data);
    pause
    hold on
end
hold off
clf

% Musique du jeu
stop(p)
[y,fs] = audioread('rizTruelle.wav');
p = audioplayer(y,fs);
play(p);


%% DONNEES GRAPHIQUES
% Fond du jeu
axes('position',[0 0 1 1]);
data = imread('fond.png');
imagesc(data);

% Controles claviers et souris
set(gcf,'KeyPressFcn',{@keyPressFcn});
set(gcf,'WindowButtonDownFcn',{@mousseDownFct});

% Flammes des enfers
fire  = 0;
uicontrol(gcf,'Style','text','BackgroundColor','w','FontSize',20,...
    'units','normalized','Position', [0.05 0.05 0.1 0.05],'String','FIRE');
Hfire = uicontrol(gcf,'Style','slider','BackgroundColor','w',...
    'units','normalized','Position', [0.15 0.05 0.3 0.05],'Value',fire);

% Temperature de la mort 
temp = 0;
Hcuis = uicontrol(gcf,'String','T°','Style','text','BackgroundColor','r',...
    'units','normalized','Position', [0.08 0.3 0.1 0.05],'FontSize',20);
Htemp = uicontrol(gcf,'Style','slider','Value',temp,'BackgroundColor','w',...
    'units','normalized','Position', [0.18 0.3 0.27 0.05]);

% Cuillere pour touiller
cuil = 0;
uicontrol(gcf,'String','BLEND','Style','text','BackgroundColor','w',...
    'units','normalized','Position', [0.05 0.66 0.1 0.05],'FontSize',20);
Hcuil = uicontrol(gcf,'Style','slider','Value',cuil,'BackgroundColor','w',...
    'units','normalized','Position', [0.15 0.66 0.3 0.05]);

% Apettit du sumo
sumo = 0.4;
uicontrol(gcf,'String','SUMO-ZA','Style','text','BackgroundColor','w',...
    'units','normalized','Position', [0.575 0.05 0.1 0.05],'FontSize',20);
Hsumo = uicontrol(gcf,'Style','slider','Value',sumo,'BackgroundColor','w',...
    'units','normalized','Position', [0.675 0.05 0.25 0.05]);

% Poubelle a riz trop cuit 
Hgarb = uicontrol(gcf,'String',' TRASH','Style','checkbox',...
    'Value',0,'BackgroundColor','r','FontSize',20,...
    'units','normalized','Position', [0.28 0.8 0.1 0.05]);

% Sac bonus pour l'ajout de riz
uicontrol('Style','text','BackgroundColor','w',...
    'units','normalized','Position',[0.55 0.82 0.04 0.04],...
    'String','+4','FontSize',20);
uicontrol('Style','text','BackgroundColor','w',...
    'units','normalized','Position',[0.89 0.82 0.04 0.04],...
    'String','+6','FontSize',20);

% Grains de riz de cuisson
Nriz = 10;
axes('position',[0 0 1 1]);
hold on
for i = 1:Nriz
   Hrice{i} = plot(0.1+0.3*i/Nriz,0.5,'ko','MarkerSize',20,...
       'MarkerFaceColor','k','LineWidth',2,'Userdata',-1);
end

% Grain de tir
Htir{1} = plot(0.5,0.5,'ko','MarkerSize',20,'LineWidth',2,...
    'MarkerFaceColor','w','Visible','off');   % positikon
Htir{2} = [0 0];                              % vitesse 

% Direction du vent aleatoire toute les 10 secondes
Hwind  = quiver(0.74,0.962,0,0,'w','LineWidth',10);

% Configuration graphique
hold off 
axis([0 1 0 1])
axis off


%% MOTEUR PHYSIQUE EQUATION DE LA CHALEUR
% http://www.math.u-psud.fr/~fdubois/cours/idef/idef-ch4.pdf
% Parametres
N     = 100;    % Nombre de points
L     = 1;      % Hauteur de l'eau dans la casserole
alpha = 10;     % Diffusion
theta = 1;      % Conditioon du theta shema ( 1/2 <= theta <= 1)
dt    = 1e-2;   % Pas de temps 

% Discretisation en espace
Z = L*(0:N-1)'/(N-1);

% Matrice du theta schema
e = ones(N,1);
A = spdiags([-theta*alpha*e 1+2*theta*alpha*e -theta*alpha*e],-1:1,N,N);
B = spdiags([(1-theta)*alpha*e 1-2*(1-theta)*alpha*e (1-theta)*alpha*e],-1:1,N,N);

% Condition de bord (Dirichlet)
% T0 : Temperateure imposee a gauche par le feu
% T1 : Temperature imposee a droite par la convection de la cuillere
Tbord = @(T0,TL) [alpha*(theta*T0 + (1-theta)*T0); ...
    zeros(N-2,1);...
    alpha*(theta*TL + (1-theta)*TL)];

% Temperature initiale nulle
T = zeros(N,1); 

% % Representation graphique 1D
% At = axes('position',[0.6  0.2  0.3 0.6]);
% Ht = plot(At,Z,T,'-o');
% grid on
% axis([0 L 0 1])



%% INITIALISATION DU JEU
% Initialisation controles clavier
KEYcuil = 0;    % Par default pas de muovement de cuillere
KEYfire = 0;    % Par default bloqué a 0
KEYgarb = 0;    % Par default poubelle fermee
KEYtemp = 1;    % Par default deplacement autorise

% Grains de riz au depart
NrizStart = 4; 
for i = 1:NrizStart
    set(Hrice{i},'MarkerFaceColor','w','UserData',0.5*rand) 
end

% Timers
Htmr5 = timer('TimerFcn',@windForce,'ExecutionMode','fixedSpacing',...
    'Period',5);
start(Htmr5)
Hgarg = timer('TimerFcn',@gargouillis,'ExecutionMode','fixedSpacing',...
    'Period',17);
start(Hgarg)

% Initialisation controle souris
Irice = 0;     % Indice du grain de riz selectione

% Riz grilles
rizGrill = 1.1 + 0.5*rand(Nriz,1);

% Moteur physique de tir par PFD (sum F = m*a) avec integration
% infinitesimale
g    = 1;            % Gravite
Ftir = 120;          % Force du tir
Fwind  = -1+2*rand;  % Force du vent


%% BOUCLE DE JEU INFINIE
% Evolution en temps
while 1
    % Temperature au milieu de la casserolle incluse dans [0 1]
    tmp = min(1,1.5*T(N/2));    
    set(Htemp,'Value',tmp);
    
    % Gestion de chaque grain de riz
    for i = 1:Nriz
        % Etat
        etat = get(Hrice{i},'UserData');
        
        % Si le grain de riz existe
        if etat >= 0
            % Cuisson
            if tmp <= 1/2      % Pas de cuisson
                
            elseif tmp <= 4/5  % Cuisson moyenne  
                etat = etat + dt/10;
            else               % Cuisson forte (2x plus rapide)
                etat = etat + dt/5;                
            end
        
            % Couleur de la cuisson
            if etat >= rizGrill(i)     % Ce riz est trop cuit
                etat = 2;
                set(Hrice{i},'MarkerFaceColor','r');
            elseif etat >= 1             % Riz bien cuit
                set(Hrice{i},'MarkerFaceColor','g');
            else
                set(Hrice{i},'MarkerFaceColor','w');
            end
            
            % Sauvegarde
            set(Hrice{i},'UserData',etat)
        
        % Le grain de riz n'existe pas   
        else
            set(Hrice{i},'UserData',-1,'MarkerFaceColor','k','Color','k')
        end
    end           
    
    % Controle clavier de la temperature avec variations lineaires
    if KEYfire == -1           % Fleche gauche : diminue le feu
        fire = max(0,fire - dt);
    elseif KEYfire == 1        % Fleche droite : augmente le feu
        fire = min(1,fire + dt);
    end
    
    % Controle clavier de la cuillere
    if KEYcuil
        cuil = min(1,cuil + 1/8);
        KEYcuil = 0;
    else
        cuil = max(0,cuil - 1e-3);
    end
    
    % Temperature maximale ou cuillere tourne trop fort 
    %   => eau deborde et eteint le feu
    if tmp == 1 || cuil == 1
        fire = 0;
        KEYtemp = 0;
        KEYfire = -1;
    elseif tmp < 0.1
        KEYtemp = 1;
    end   
  
    % Si il y a des grains tires
    if strcmp(get(Htir{1},'Visible'),'on')
        % Position et vitesse du grain
        x = [get(Htir{1},'XData') , get(Htir{1},'YData')];
        v = Htir{2};
        
        % Imposition de la vitesse initiale par la catapulte
        if sum(v) == 0
            % Absice normalisee
            xnrm = (-0.55 + x(1))/0.4;
            
            % Vitesse initiale en fonction de la position sur la cata
            v = Ftir*[-sin(0.2*(-1+2*xnrm)) 1]*dt;
        end
        
        % Integration de l'acceleration (PFD) : a = dv/dt
        v = v + [0 -g*dt] + [Fwind*dt 0];
        
        % Integration de la vitesse : x = dv/dt;
        x = x + v*dt;
        
        % Eteindres grains sortis du cadre
        if min(x) < 0 || max(x) > 1
            set(Htir{1},'Visible','off')
        
        % Nourir le sumo    
        elseif abs(x(1)-0.745) < 0.04 && abs(x(2)-0.59) < 0.05  
            sumo = sumo - 1/6.5;
            set(Htir{1},'Visible','off')
            sound(SUMOmange,44100)
        
        % Recharger le riz 
        elseif abs(x(1)-0.57) < 0.07 && abs(x(2)-0.85) < 0.07 || ...
                abs(x(1)-0.91) < 0.07 && abs(x(2)-0.85) < 0.07
            % Bonus + 2 ou + 5
            if abs(x(1)-0.57) < 0.07
                bonus = 4;
            else
                bonus = 6;
            end
            
            % Recharge
            k = 1;            
            for i = 1:Nriz
                if get(Hrice{i},'UserData') == -1
                    set(Hrice{i},'UserData',0.5*rand);
                    k = k+1;
                end
                if k > bonus
                    break
                end
            end
            set(Htir{1},'Visible','off')
            sound(BONUS,48000)
            
        % Mise a jour vitesse et graphique
        else
            set(Htir{1},'XData',x(1),'YData',x(2));
            Htir{2} = v;
        end
    end
    
    % Apetit du sumo, augentation de 0 a 1 en 40 secondes 
    sumo = min(1,max(0,sumo + dt/tMax));    
    
    % Mise a jour temperature dans la casserole avec conditions de bord
    V = Tbord(fire,max(0,fire-cuil));
    T = A\(B*T+V);       
%     set(Ht,'YData',T)

    % Mise a jour graphique des interfaces
    set(Hfire,'Value',fire); 
    if tmp <= 1/2
        set(Hcuis,'BackgroundColor','r','Value',cuil);
    elseif tmp <= 4/5
        set(Hcuis,'BackgroundColor','y','Value',cuil);
    else
        set(Hcuis,'BackgroundColor','g','Value',cuil)
    end
    set(Hcuil,'Value',cuil);
    if KEYgarb == 1
        set(Hgarb,'Value',1,'BackgroundColor','g');
    else
        set(Hgarb,'Value',0,'BackgroundColor','r'); 
    end
    set(Hwind,'UData',0.25*Fwind)  
    set(Hsumo,'Value',sumo);
    
    % Fin du jeu avec diapo
    if sumo == 1 % defaite
        clf
        axes('position',[0 0 1 1]);
        data = imread('defaite.png');
        imagesc(data)
        sound(DEFAITE,48000) 
        pause(5)
        break

    elseif sumo == 0 % victoire
        clf
        axes('position',[0 0 1 1]);
        data = imread('victoire.png');
        imagesc(data);
        sound(VICTORY,44100)
        pause(5)
        break
    end
        
    % Pas de temps
    pause(dt)
end

% Credits
data = imread('credits.png');
imagesc(data)
end


function keyPressFcn(~,~)
global KEYcuil
global KEYfire
global KEYgarb
global KEYtemp
global POUBouv

% Touche presse
key = double(get(gcf,'Currentcharacter'));

% Poubelle
if key == 13
    sound(POUBouv,48000)
    KEYgarb = 1;
    Tgarb2 = timer('TimerFcn',@closeGarbage,'ExecutionMode','singleShot',...
        'StartDelay',3);
    start(Tgarb2)
end

% Temperature
if KEYtemp && key == 28     % Diminution de la temperature si autorise
    KEYfire = -1;
elseif KEYtemp && key == 29 % Augmentation de la temperature si autorise
    KEYfire = 1;
end

% Cuillere
if key == 32 
    KEYcuil = 1;
end
end


function mousseDownFct(~,~)
global KEYgarb
global Hrice
global Irice
global Htir
global CORDE

% Position souris
siz = get(gcf,'Position');
pos = get(gcf,'CurrentPoint');
pos = [pos(1)/siz(3),pos(2)/siz(4)];

% Mise a la poubelle
if Irice && KEYgarb && abs(pos(1)-0.433) < 0.04 && abs(pos(2)-0.83) < 0.1
    % Extinction grain
    set(Hrice{Irice},'UserData',-1)
    Irice = 0;
   
% Lache sur la corde des grains de riz selectionnes pas trop cuit
elseif Irice && get(Hrice{Irice},'UserData') < 2 && ... 
        abs(pos(1)-0.75) < 0.19 && abs(pos(2)-0.19) < 0.03
    set(Htir{1},'Visible','on','XData',pos(1),'YData',pos(2),...
        'MarkerFaceColor',get(Hrice{Irice},'MarkerFaceColor'));
    Htir{2} = [0 0];             % vitesse nulle
    set(Hrice{Irice},'UserData',-1);
    Irice = 0;
    sound(CORDE,44100)
        
% Recherche d'un grain de riz associe    
else    
    % Initialisation de l'indice du riz
    Irice = 0;
    for i = 1:length(Hrice)
        % Coordonnees et etat de chaque grain de riz
        x    = get(Hrice{i},'XData');
        y    = get(Hrice{i},'YData');
        etat = get(Hrice{i},'UserData');
        
        % Positions coincidantes => selection
        if etat >=1 && abs(pos(1)-x)<0.015 && abs(pos(2)-y)<0.015
            Irice = i;
            set(Hrice{i},'Color','y','LineWidth',2)
            
        % Positions non coincidantes => nettoyage
        else
            set(Hrice{i},'Color','k','LineWidth',2)
        end
    end 
end
end


function windForce(~,~)
global Fwind
Fwind = -1+2*rand;
end


function closeGarbage(~,~)
global KEYgarb
global POUBfer
KEYgarb = 0;
sound(POUBfer,44100)
end

function gargouillis(~,~)
global GARGOUIL
sound(GARGOUIL,48000)
end

